@@ -1,6 +1,6 @@
 /* mpfr_log10 -- logarithm in base 10.
 
-Copyright 2001-2019 Free Software Foundation, Inc.
+Copyright 2001-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -129,7 +129,7 @@ mpfr_log10 (mpfr_ptr r, mpfr_srcptr a, mpfr_rnd_t rnd_mode)
         if (MPFR_IS_POS (t)
             && mpfr_integer_p (t) && mpfr_fits_ulong_p (t, MPFR_RNDN)
             && !mpfr_ui_pow_ui (tt, 10, mpfr_get_ui (t, MPFR_RNDN), MPFR_RNDN)
-            && mpfr_cmp (a, tt) == 0)
+            && mpfr_equal_p (a, tt))
           break;
 
         /* actualization of the precision */
